@@ -1,5 +1,5 @@
 /*
-*   $Id: c.c,v 1.34 2006/05/30 04:37:11 darren Exp $
+*   $Id: c.c 614 2007-08-19 22:46:13Z elliotth $
 *
 *   Copyright (c) 1996-2003, Darren Hiebert
 *
@@ -30,9 +30,7 @@
 /*
 *   MACROS
 */
-#define MaxFields   6
 
-#define stringValue(a)      #a
 #define activeToken(st)     ((st)->token [(int) (st)->tokenIndex])
 #define parentDecl(st)      ((st)->parent == NULL ? \
                             DECL_NONE : (st)->parent->declaration)
@@ -287,7 +285,7 @@ static kindOption CKinds [] = {
 	{ TRUE,  't', "typedef",    "typedefs"},
 	{ TRUE,  'u', "union",      "union names"},
 	{ TRUE,  'v', "variable",   "variable definitions"},
-	{ FALSE, 'x', "externvar",  "external variable declarations"},
+	{ FALSE, 'x', "externvar",  "external and forward variable declarations"},
 };
 
 typedef enum {
@@ -316,17 +314,19 @@ static kindOption CsharpKinds [] = {
 /* Used to index into the JavaKinds table. */
 typedef enum {
 	JK_UNDEFINED = -1,
-	JK_CLASS, JK_FIELD, JK_INTERFACE, JK_LOCAL, JK_METHOD,
-	JK_PACKAGE, JK_ACCESS, JK_CLASS_PREFIX
+	JK_CLASS, JK_ENUM_CONSTANT, JK_FIELD, JK_ENUM, JK_INTERFACE,
+	JK_LOCAL, JK_METHOD, JK_PACKAGE, JK_ACCESS, JK_CLASS_PREFIX
 } javaKind;
 
 static kindOption JavaKinds [] = {
-	{ TRUE,  'c', "class",     "classes"},
-	{ TRUE,  'f', "field",     "fields"},
-	{ TRUE,  'i', "interface", "interfaces"},
-	{ FALSE, 'l', "local",     "local variables"},
-	{ TRUE,  'm', "method",    "methods"},
-	{ TRUE,  'p', "package",   "packages"},
+	{ TRUE,  'c', "class",         "classes"},
+	{ TRUE,  'e', "enum constant", "enum constants"},
+	{ TRUE,  'f', "field",         "fields"},
+	{ TRUE,  'g', "enum",          "enum types"},
+	{ TRUE,  'i', "interface",     "interfaces"},
+	{ FALSE, 'l', "local",         "local variables"},
+	{ TRUE,  'm', "method",        "methods"},
+	{ TRUE,  'p', "package",       "packages"},
 };
 
 /* Used to index into the VeraKinds table. */
@@ -382,7 +382,7 @@ static const keywordDesc KeywordTable [] = {
 	{ "delete",         KEYWORD_DELETE,         { 0, 1, 0, 0, 0 } },
 	{ "double",         KEYWORD_DOUBLE,         { 1, 1, 1, 1, 0 } },
 	{ "else",           KEYWORD_ELSE,           { 1, 1, 0, 1, 0 } },
-	{ "enum",           KEYWORD_ENUM,           { 1, 1, 1, 0, 1 } },
+	{ "enum",           KEYWORD_ENUM,           { 1, 1, 1, 1, 1 } },
 	{ "event",          KEYWORD_EVENT,          { 0, 0, 1, 0, 1 } },
 	{ "explicit",       KEYWORD_EXPLICIT,       { 0, 1, 1, 0, 0 } },
 	{ "extends",        KEYWORD_EXTENDS,        { 0, 0, 0, 1, 1 } },
@@ -713,8 +713,9 @@ static void initMemberInfo (statementInfo *const st)
 	if (st->parent != NULL) switch (st->parent->declaration)
 	{
 		case DECL_ENUM:
+			accessDefault = (isLanguage (Lang_java) ? ACCESS_PUBLIC : ACCESS_UNDEFINED);
+			break;
 		case DECL_NAMESPACE:
-		case DECL_UNION:
 			accessDefault = ACCESS_UNDEFINED;
 			break;
 
@@ -727,6 +728,7 @@ static void initMemberInfo (statementInfo *const st)
 
 		case DECL_INTERFACE:
 		case DECL_STRUCT:
+		case DECL_UNION:
 			accessDefault = ACCESS_PUBLIC;
 			break;
 
@@ -843,12 +845,14 @@ static javaKind javaTagKind (const tagType type)
 	javaKind result = JK_UNDEFINED;
 	switch (type)
 	{
-		case TAG_CLASS:     result = JK_CLASS;     break;
-		case TAG_FIELD:     result = JK_FIELD;     break;
-		case TAG_INTERFACE: result = JK_INTERFACE; break;
-		case TAG_LOCAL:     result = JK_LOCAL;     break;
-		case TAG_METHOD:    result = JK_METHOD;    break;
-		case TAG_PACKAGE:   result = JK_PACKAGE;   break;
+		case TAG_CLASS:      result = JK_CLASS;         break;
+		case TAG_ENUM:       result = JK_ENUM;          break;
+		case TAG_ENUMERATOR: result = JK_ENUM_CONSTANT; break;
+		case TAG_FIELD:      result = JK_FIELD;         break;
+		case TAG_INTERFACE:  result = JK_INTERFACE;     break;
+		case TAG_LOCAL:      result = JK_LOCAL;         break;
+		case TAG_METHOD:     result = JK_METHOD;        break;
+		case TAG_PACKAGE:    result = JK_PACKAGE;       break;
 
 		default: Assert ("Bad Java tag type" == NULL); break;
 	}
@@ -1466,6 +1470,14 @@ static void readIdentifier (tokenInfo *const token, const int firstChar)
 
 	initToken (token);
 
+	/* Bug #1585745: strangely, C++ destructors allow whitespace between
+	 * the ~ and the class name. */
+	if (isLanguage (Lang_cpp) && firstChar == '~')
+	{
+		vStringPut (name, c);
+		c = skipToNonWhite ();
+	}
+
 	do
 	{
 		vStringPut (name, c);
@@ -1476,7 +1488,7 @@ static void readIdentifier (tokenInfo *const token, const int firstChar)
 			first = FALSE;
 		}
 		c = cppGetc ();
-	} while (isident (c) || (isLanguage (Lang_java) && isHighChar (c)));
+	} while (isident (c) || (isLanguage (Lang_java) && (isHighChar (c) || c == '.')));
 	vStringTerminate (name);
 	cppUngetc (c);        /* unget non-identifier character */
 
@@ -1499,15 +1511,25 @@ static void readPackageName (tokenInfo *const token, const int firstChar)
 	cppUngetc (c);        /* unget non-package character */
 }
 
-static void readPackage (statementInfo *const st)
+static void readPackageOrNamespace (statementInfo *const st, const declType declaration)
 {
-	tokenInfo *const token = activeToken (st);
-	Assert (isType (token, TOKEN_KEYWORD));
-	readPackageName (token, skipToNonWhite ());
-	token->type = TOKEN_NAME;
-	st->declaration = DECL_PACKAGE;
-	st->gotName = TRUE;
-	st->haveQualifyingName = TRUE;
+	st->declaration = declaration;
+	
+	if (declaration == DECL_NAMESPACE && !isLanguage (Lang_csharp))
+	{
+		/* In C++ a namespace is specified one level at a time. */
+		return;
+	}
+	else
+	{
+		/* In C#, a namespace can also be specified like a Java package name. */
+		tokenInfo *const token = activeToken (st);
+		Assert (isType (token, TOKEN_KEYWORD));
+		readPackageName (token, skipToNonWhite ());
+		token->type = TOKEN_NAME;
+		st->gotName = TRUE;
+		st->haveQualifyingName = TRUE;
+	}
 }
 
 static void processName (statementInfo *const st)
@@ -1682,6 +1704,11 @@ static void skipStatement (statementInfo *const st)
 	skipToOneOf (";");
 }
 
+static void processInterface (statementInfo *const st)
+{
+	st->declaration = DECL_INTERFACE;
+}
+
 static void processToken (tokenInfo *const token, statementInfo *const st)
 {
 	switch (token->keyword)        /* is it a reserved word? */
@@ -1710,12 +1737,10 @@ static void processToken (tokenInfo *const token, statementInfo *const st)
 		case KEYWORD_IMPORT:    skipStatement (st);                     break;
 		case KEYWORD_INT:       st->declaration = DECL_BASE;            break;
 		case KEYWORD_INTEGER:   st->declaration = DECL_BASE;            break;
-		case KEYWORD_INTERFACE: st->declaration = DECL_INTERFACE;       break;
+		case KEYWORD_INTERFACE: processInterface (st);                  break;
 		case KEYWORD_LOCAL:     setAccess (st, ACCESS_LOCAL);           break;
 		case KEYWORD_LONG:      st->declaration = DECL_BASE;            break;
-		case KEYWORD_NAMESPACE: st->declaration = DECL_NAMESPACE;       break;
 		case KEYWORD_OPERATOR:  readOperator (st);                      break;
-		case KEYWORD_PACKAGE:   readPackage (st);                       break;
 		case KEYWORD_PRIVATE:   setAccess (st, ACCESS_PRIVATE);         break;
 		case KEYWORD_PROGRAM:   st->declaration = DECL_PROGRAM;         break;
 		case KEYWORD_PROTECTED: setAccess (st, ACCESS_PROTECTED);       break;
@@ -1733,7 +1758,11 @@ static void processToken (tokenInfo *const token, statementInfo *const st)
 		case KEYWORD_VOID:      st->declaration = DECL_BASE;            break;
 		case KEYWORD_VOLATILE:  st->declaration = DECL_BASE;            break;
 		case KEYWORD_VIRTUAL:   st->implementation = IMP_VIRTUAL;       break;
-
+		case KEYWORD_WCHAR_T:   st->declaration = DECL_BASE;            break;
+		
+		case KEYWORD_NAMESPACE: readPackageOrNamespace (st, DECL_NAMESPACE); break;
+		case KEYWORD_PACKAGE:   readPackageOrNamespace (st, DECL_PACKAGE);   break;
+		
 		case KEYWORD_EVENT:
 			if (isLanguage (Lang_csharp))
 				st->declaration = DECL_EVENT;
@@ -2018,6 +2047,26 @@ static void analyzePostParens (statementInfo *const st, parenInfo *const info)
 	}
 }
 
+static void processAngleBracket (void)
+{
+	int c = cppGetc ();
+	if (c == '>') {
+		/* already found match for template */
+	} else if ((isLanguage (Lang_cpp) || isLanguage (Lang_java)) && c != '<' && c != '=') {
+		/* this is a template */
+		cppUngetc (c);
+		skipToMatch ("<>");
+	} else if (c == '<') {
+		/* skip "<<" or "<<=". */
+		c = cppGetc ();
+		if (c != '=') {
+			cppUngetc (c);
+		}
+	} else {
+		cppUngetc (c);
+	}
+}
+
 static int parseParens (statementInfo *const st, parenInfo *const info)
 {
 	tokenInfo *const token = activeToken (st);
@@ -2098,7 +2147,7 @@ static int parseParens (statementInfo *const st, parenInfo *const info)
 
 			case '<':
 				info->isKnrParamList = FALSE;
-				skipToMatch ("<>");
+				processAngleBracket ();
 				break;
 
 			case ')':
@@ -2311,17 +2360,6 @@ static void processColon (statementInfo *const st)
 	}
 }
 
-static void processAngleBracket (void)
-{
-	int c = cppGetc ();
-	if (c == '>')
-		;  /* already found match for template */
-	else if (isLanguage (Lang_cpp) &&  c != '<' && c != '=')
-		skipToMatch ("<>");        /* this is a template */
-	else
-		cppUngetc (c);
-}
-
 /*  Skips over any initializing value which may follow an '=' character in a
  *  variable definition.
  */
@@ -2349,6 +2387,7 @@ static int skipInitializer (statementInfo *const st)
 			case '[': skipToMatch ("[]"); break;
 			case '(': skipToMatch ("()"); break;
 			case '{': skipToMatch ("{}"); break;
+			case '<': processAngleBracket(); break;
 
 			case '}':
 				if (insideEnumBody (st))
@@ -2381,7 +2420,7 @@ static void processInitializer (statementInfo *const st)
 			setToken (st, TOKEN_SEMICOLON);
 		else if (c == ',')
 			setToken (st, TOKEN_COMMA);
-		else if ('}'  &&  inEnumBody)
+		else if (c == '}'  &&  inEnumBody)
 		{
 			cppUngetc (c);
 			setToken (st, TOKEN_COMMA);
@@ -2400,10 +2439,35 @@ static void parseIdentifier (statementInfo *const st, const int c)
 		processToken (token, st);
 }
 
+static void parseJavaAnnotation (statementInfo *const st)
+{
+	/*
+	 * @Override
+	 * @Target(ElementType.METHOD)
+	 * @SuppressWarnings(value = "unchecked")
+	 *
+	 * But watch out for "@interface"!
+	 */
+	tokenInfo *const token = activeToken (st);
+	
+	int c = skipToNonWhite ();
+	readIdentifier (token, c);
+	if (token->keyword == KEYWORD_INTERFACE)
+	{
+		/* Oops. This was actually "@interface" defining a new annotation. */
+		processInterface (st);
+	}
+	else
+	{
+		/* Bug #1691412: skip any annotation arguments. */
+		skipParens ();
+	}
+}
+
 static void parseGeneralToken (statementInfo *const st, const int c)
 {
 	const tokenInfo *const prev = prevToken (st, 1);
-
+	
 	if (isident1 (c) || (isLanguage (Lang_java) && isHighChar (c)))
 	{
 		parseIdentifier (st, c);
@@ -2430,6 +2494,10 @@ static void parseGeneralToken (statementInfo *const st, const int c)
 		if (c2 != '=')
 			cppUngetc (c2);
 	}
+	else if (c == '@' && isLanguage (Lang_java))
+	{
+		parseJavaAnnotation (st);
+	}
 	else if (isExternCDecl (st, c))
 	{
 		st->declaration = DECL_NOMANGLE;
@@ -2639,7 +2707,7 @@ static void tagCheck (statementInfo *const st)
 			else if (isType (prev, TOKEN_NAME))
 			{
 				if (isContextualKeyword (prev2))
-					st->scope = SCOPE_EXTERN;
+					makeTag (prev, st, TRUE, TAG_EXTERN_VAR);
 				else
 					qualifyVariableTag (st, prev);
 			}
@@ -2650,6 +2718,13 @@ static void tagCheck (statementInfo *const st)
 				else
 					qualifyFunctionDeclTag (st, prev2);
 			}
+			if (isLanguage (Lang_java) && token->type == TOKEN_SEMICOLON && insideEnumBody (st))
+			{
+				/* In Java, after an initial enum-like part,
+				 * a semicolon introduces a class-like part.
+				 * See Bug #1730485 for the full rationale. */
+				st->parent->declaration = DECL_CLASS;
+			}
 			break;
 
 		default: break;
@@ -2705,7 +2780,7 @@ static boolean findCTags (const unsigned int passCount)
 	boolean retry;
 
 	Assert (passCount < 3);
-	cppInit ((boolean) (passCount > 1));
+	cppInit ((boolean) (passCount > 1), isLanguage (Lang_csharp));
 	Signature = vStringNew ();
 
 	exception = (exception_t) setjmp (Exception);
